%--- New Keynesian Phillips Curve - baseline model.
%--- Confidence Regions using GMM weak instruments tests, Split sample 
%--- test (S, KLM,JKLM, KJ, CLR) and generalized S (sup, ave, exp, qll_10)
%--- We concentrate out the constant before implementing the test.
%--- See note: "Identification using stability restrictions: supplement "

%---Need the Genetic Algorithm Toolbox to compute the Split-CLR test

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%.%        Hybrid  Model - baseline model                            %.%
%.% pi_t -Vrho*pi_{t-1} = beta*pi_{t+1}+ lambda*mc_t + kappa + e     %.%
%.%                                                                  %.%
%.% lambda = (1-alpha)(1-alpha*beta)[alpha(1+beta*Vrho)]^{-1}        %.%
%.%                                                                  %.%
%.% when beta =1; the moment condition is:                           %.%
%.% f_t = Z_t'*e_t                                                   %.%
%.%  e_t= (Vrho*diff(pi_t) - diff(pi_{t+1}) - gamma*xtilde_t - kappa)%.%                                    
%.% gamma = (1-alpha)^2/alpha                                        %.%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

tic

%--- Add the folder functions and table in the search path
addpath('.../functions','-end')
addpath('.../tables','-end')

nkpc =importdata('nkpc_data.mat');	
range=[1984.1 2010.4];
%---adjustment factor labor share: upsilon = (a*(mu-1))/(mu-a);
mu = 1.2;  % markup;      
a  = 2/3;  % labor share - Cobb Douglas
%--- Parameter Space Grid Search Points   
Alpha   = 0.01:0.01:.999;   
l.alpha = length(Alpha);
Vrho    = 0:0.01:1;                        
l.vrho  = length(Vrho);
%---load the data: 
[X,Y,Z,T,yearq] = data_nkpc(nkpc,range,mu,a);

%---moment condition: Z'*(Y*b-X*c)
Xp      = X'; 
Zp      = Z';
G       = Zp*X;
Gp      = G';

%---Dimensions  
p       = 2;           % # of tested structural parameters
k.y     = size(Y,2);   % # of variables
k.x     = size(X,2);   % # of included instruments
k.z     = size(Z,2);   % # of instruments (total)
k.zmx   = k.z-k.x;     % # of excluded instruments
k.zz    = 2*k.z;
k.yy    = 2*k.y;
k.zzmx  = k.zz-k.x;
k.zzmxx = k.zz-2*k.x;
k.zy    = k.z*k.y;

%--- trimming paramter:
trim  = 0.15;    
t.l   = ceil(trim*T); t.u =floor((1-trim)*T);
%--- terms for qLL_AR test
c10 = 10; term_c10= c10/(1+c10);
%--- Option for implementing the Genetic Algorithm function ga
options=gaoptimset('Display','off','Generations', 250,'HybridFcn',@fminsearch);
%---Auxiliary Projection Matrix
I.T = eye(T);
Mx  = I.T - X*((Xp*X)\(Xp)); 

%--- HAC estimate of Sigma_hat (see equation (S-8))
scale    = T/(T-k.x);
s.s      = (t.l:1:t.u)/T;
Sigma_hat= scale*HAC_Sigma_hat(Mx*Y,Z,k.y,k.z,T);

%--- Permutation Matrices (computing Var(vec(s.ZY)), see equation (S-16) 
% Per.C = permutation columns; 
% Per.R = kron(E',I.k.z);  %permutation rows
I.k.z        = eye(k.z);
[Per.C,Per.R]= permut_matrix(k.y,k.yy,I.k.z);

%--- Storage space for Weak Instruments, Split Sample, Generalized S, S-tilde statistics 
GMM   = struct('S',zeros(l.alpha,l.vrho),'KLM',zeros(l.alpha,l.vrho),...
      'JKLM',zeros(l.alpha,l.vrho),'CLR',zeros(l.alpha,l.vrho),'rk',zeros(l.alpha,l.vrho));

Split = struct('S',zeros(l.alpha,l.vrho), 'KLM',zeros(l.alpha,l.vrho),...
      'JKLM',zeros(l.alpha,l.vrho),'CLR',zeros(l.alpha,l.vrho),'rk',zeros(l.alpha,l.vrho));

GenS   = struct('ave', zeros(l.alpha,l.vrho),...
      'exp',zeros(l.alpha,l.vrho),'qLL',zeros(l.alpha,l.vrho));

GenStilde  = struct('ave', zeros(l.alpha,l.vrho),...
      'exp',zeros(l.alpha,l.vrho),'qLL',zeros(l.alpha,l.vrho));

T_break_hat= zeros(l.alpha,l.vrho);
   
%---Auxiliary Matrices
ons.kz  = ones(1,k.z);
%---Zeros Matrices
D_hat = zeros(k.zmx,p)    ; s.D_hat = zeros(k.zz-k.x,p); 
s.S   = zeros(1,t.u-t.l+1); s.Stilde= zeros(1,t.u-t.l+1); 
s.KLM = zeros(1,t.u-t.l+1); s.DVD   = zeros(1,t.u-t.l+1);
s.D   = zeros(t.u-t.l+1,k.zz-k.x,p);
s.V_D = zeros(t.u-t.l+1,(k.zz-k.x)*p,(k.zz-k.x)*p);
%---Identity matrices
I.k.zmx= eye(k.zmx); I.k.yy= eye(2*k.y);
I.k.zz = eye(k.zz) ; I.p   = eye(p)      ;  
I.k.zzmx= eye(2*k.z-k.x) ;
%% Loop: Hypothesis Testing on Parameter Space Grid Search Points 
for j=1:l.vrho
    
    vrho   = Vrho(j);  
    
    for i=1:l.alpha
        
        alpha = Alpha(i);
        omal  = 1-alpha;
        gamma = (omal^2)/alpha;
        b     = [vrho; - 1; -gamma];
        %--- Derivative w.r.t ``deep parameters''(Vrho and Alpha) 
        db    = [1    0;                 
                 0    0;
                 0  -(alpha^2-1)/(alpha^2)];
        
        b_bar = kron(b,I.k.z);
        V_ff  = b_bar'*Sigma_hat*b_bar;% Variance S and qLL-S tests see equation (S-7)
        hV_ff = sqrtm(V_ff);           % Symmetric square root
        
        Yb   = Y*b;  
        Ydb  = Y*db; 
        H    = Z*(V_ff\Zp);
        %--- Two-stage estimator of nuisance parameters
        c2_hat = (Xp*H*X)\(Xp*H*Yb);  % see equation (S-9)		
        e_hat = Yb-X*c2_hat;          % see equation (S-11)
        
        %--- Normalized moment condition
        F_hat             =(Z.*(e_hat*ons.kz))/hV_ff; % see equation (S-12) 
        GenStilde.qLL(i,j)= qLL_S_tilde(F_hat,c10);
        
        hG   = hV_ff\G; hGp=hG';
        M_hG = I.k.z - hG*(hGp*hG\hGp);
        %--- SVD of the orthogonal projection matrix
        [U, ~] = svd(M_hG);
        L   = U(1:k.z,1:k.z-k.x);  % L*L' = M_hG & L'*L = I(k.z-k.x)
        
        Xi_hat = L'*(hV_ff\Zp*Yb);
        Xi_hatp= Xi_hat';          % see statistic S_T on page 13
        
        GMM.S(i,j)  = (1/T)*(Xi_hat)'*(Xi_hat); 
        GenS.qLL(i,j)= term_c10*GMM.S(i,j) + GenStilde.qLL(i,j);
        
        %---Jacobian Matrix for KLM test
        L_tilde = hV_ff\L;

        Aux1  = kron(b,L_tilde); 
        Aux2  = kron(db,L_tilde);     % == db_bar*kron(I.p,L_tilde);
        V_xx  = I.k.zmx;              % == Aux1'*Sigma_hat*Aux1;
        V_qx  = Aux2'*Sigma_hat*Aux1;
        V_qq  = Aux2'*Sigma_hat*Aux2; 
        V_qqx = V_qq - V_qx*(V_xx\V_qx');
               
        q_hat = L_tilde'*Zp*Ydb; h = 0;
        
        for r=1:p
            D_hat(:,h+1:r)= q_hat(:,r)...
                - V_qx( k.zmx*h+1: k.zmx*r,:)*(V_xx\(Xi_hat));
            h=h+1;
        end
        D_hat = D_hat/T; 
        D_hatp= D_hat';
        
        GMM.KLM(i,j)= (1/T)*Xi_hatp*D_hat*((D_hatp*D_hat)\(D_hatp*(Xi_hat)));
        GMM.JKLM(i,j)= GMM.S(i,j)-GMM.KLM(i,j);
                
        %--- rank statistic for CLR test:
        rk          = ranktest_svd(D_hat,I.p, V_xx^(-.5),V_qqx,p-1,T);
        GMM.rk(i,j) = rk;
        GMM.CLR(i,j)= .5*(GMM.S(i,j)-rk +...
                      sqrt((GMM.S(i,j)+rk)^2 - 4*GMM.JKLM(i,j)*rk));
        
        %--- Loop for computing Split-sample S && estimation of break date      
        s.b_bar = kron(b,I.k.zz);
        s.b_barp= s.b_bar';
        for t_break  = t.l:t.u
            ts   = t_break-t.l+1;
            s.Z  = blkdiag(Z(1:t_break,:),Z(t_break+1:T,:)); % see equation (S-13)
            s.Zp = s.Z';
            s.G  = s.Zp*X; 
            st   = s.s(ts);
            
            %--- DANGER: s.Sigma_hat != var(s.Zp*Y)
            s.Sigma_hat = blkdiag(st*Sigma_hat,(1-st)*Sigma_hat);
            s.Sigma_hat=  Per.R*s.Sigma_hat*Per.C;           
            %---- Var( T^-.5 Zbar*Yb )
            s.V_ff=  s.b_barp*s.Sigma_hat*s.b_bar; % see equation (S-16)
            
            s.hV_ff= sqrtm(s.V_ff);
            s.hG   = s.hV_ff\s.G; 
            s.hGp  = s.hG';
            %--- SVD of the orthogonal projection matrix
            s.M_hG   = I.k.zz - s.hG*(s.hGp*s.hG\s.hGp);
            [s.U, ~]= svd(s.M_hG);
            s.L   = s.U(1:k.zz,1:k.zz-k.x); % s.L*s.L' = s.M_hG                
                                            % s.L'*s.L = I(2*k.z-k.x)   
            s.X_T    = s.hV_ff\s.Zp*Yb;
            s.Xi_hat = s.L'*s.X_T;
            s.Xi_hatp= s.Xi_hat';
			
            s.S(ts)      = (1/T)*s.Xi_hatp*(s.Xi_hat);
            s.Stilde(ts) = s.S(ts)-GMM.S(i,j);

            s.L_tilde = s.hV_ff\s.L;
            s.Aux1    = kron( b,s.L_tilde);
            s.Aux2    = kron(db,s.L_tilde);
            
            s.V_xx = I.k.zzmx;               % == s.Aux1'*s.Sigma_hat*s.Aux1; 
            s.V_qx = s.Aux2'*s.Sigma_hat*s.Aux1;
            s.V_qq = s.Aux2'*s.Sigma_hat*s.Aux2; 
            s.V_qqx= s.V_qq - s.V_qx*(s.V_xx\s.V_qx');
            
            s.q_hat= s.L_tilde'*s.Zp*Ydb; h = 0;
            
            for r=1:p
                s.D_hat(:,h+1:r)= s.q_hat(:,r)-...
                               s.V_qx((k.zzmx)*h+1:(k.zzmx)*r,:)...
                               *(s.V_xx\s.Xi_hat);
                h=h+1;
            end
            s.D_hat  = s.D_hat/T;
            s.D_hatp = s.D_hat';
            
            s.KLM(ts)=(1/T)*...
                 s.Xi_hatp*s.D_hat*((s.D_hatp*s.D_hat)\(s.D_hatp*s.Xi_hat));
            s.DVD(ts)= s.D_hat(:)'*(s.V_qqx\s.D_hat(:));
            
            % Saved s.D_hat and s.V_qqx to be used in computing split CLR  
            s.D(ts,:,:)  = s.D_hat;
            s.V_D(ts,:,:)= s.V_qqx;
        end
        GenStilde.ave(i,j) = mean(s.Stilde);
        GenStilde.exp(i,j) = log(mean(exp(.5*s.Stilde)));
        
        GenS.ave(i,j) = mean(s.S);
        GenS.exp(i,j) = log(mean(exp(.5*s.S)));
        %---Estimating the break date
        [~, ir]           = sort(s.DVD,'descend');
        t_break_hat       = t.l+ir(1)-1; 
        T_break_hat(i,j)  = yearq(t_break_hat,1);
        
        Split.S(i,j)   = s.S(ir(1));
        Split.KLM(i,j) = s.KLM(ir(1));     
        Split.JKLM(i,j)= Split.S(i,j)-Split.KLM(i,j);
        
        sD_hat = squeeze(s.D(ir(1),:,:));
        sV_qqx = squeeze(s.V_D(ir(1),:,:));
        
        [xmin,rkt]    = ga(@(phi) rankmin(phi,sD_hat,sV_qqx,I.k.zzmx,T),p-1,options);
        Split.rk(i,j) = rkt;
        
        Split.CLR(i,j)= .5*(Split.S(i,j)-rkt+ ...
                sqrt((Split.S(i,j) + rkt)^2 - 4*Split.JKLM(i,j)*rkt));   
    end
end

%% Computing the tests p-values 

%--- P-values: Weak Instruments Tests
CR_GMM = struct(...
        'S',chi2cdf(GMM.S(:,:),k.z-k.x), 'KLM',chi2cdf(GMM.KLM(:,:),p),...
         'JKLM',chi2cdf(GMM.JKLM(:,:),k.z-k.x-p),'CLR',zeros(l.alpha,l.vrho));

%--- P-values: Split Sample Tests
CR_Split = struct(...
        'S',chi2cdf(Split.S(:,:),k.zz-k.x), 'KLM',chi2cdf(Split.KLM(:,:),p),...
         'JKLM',chi2cdf(Split.JKLM(:,:),k.zz-k.x-p),'CLR',zeros(l.alpha,l.vrho));

%--- P-values: Generalized-S and Stability-S Tests
CR_GenS     = struct('ave',zeros(l.alpha,l.vrho),...
                'exp',zeros(l.alpha,l.vrho),'qLL',zeros(l.alpha,l.vrho));
CR_GenStilde= struct('ave',zeros(l.alpha,l.vrho),...
                'exp',zeros(l.alpha,l.vrho),'qLL',zeros(l.alpha,l.vrho));

%--- P-values: CLR- and Split-CLR Tests
P_values1    = pvalue_CLR(p,k.z-k.x,l.alpha*l.vrho,GMM.CLR(:),GMM.rk(:));
CR_GMM.CLR   = reshape(P_values1, [l.alpha l.vrho]);

P_values2    = pvalue_CLR(p,k.zz-k.x,l.alpha*l.vrho,Split.CLR(:),Split.rk(:));
CR_Split.CLR = reshape(P_values2, [l.alpha l.vrho]);

%--- P-values: Generalized-S and S_tilde Tests
[P_values_genS, P_values_genStilde] = pvalue_GenS(k.z,k.z-k.x,l.alpha*l.vrho,GenS,GenStilde);
CR_GenS.ave = reshape(P_values_genS(:,1),[l.alpha l.vrho]);
CR_GenS.exp = reshape(P_values_genS(:,2),[l.alpha l.vrho]);
CR_GenS.qLL = reshape(P_values_genS(:,3),[l.alpha l.vrho]);

CR_GenStilde.ave = reshape(P_values_genStilde(:,1),[l.alpha l.vrho]);
CR_GenStilde.exp = reshape(P_values_genStilde(:,2),[l.alpha l.vrho]);
CR_GenStilde.qLL = reshape(P_values_genStilde(:,3),[l.alpha l.vrho]);

%--- Save the values for confidence regions plot  
save data_nkpc_baseline_8410 Vrho Alpha CR_* T_break_hat 

%% Confidence Regions
Color1=importdata('Color1.mat');

CR_values = [CR_GMM.S(:,:) CR_Split.S(:,:) CR_GMM.CLR(:,:) CR_Split.CLR(:,:) ...
            CR_GenS.ave(:,:) CR_GenStilde.ave(:,:) CR_GenS.exp(:,:)...
            CR_GenStilde.exp(:,:) CR_GenS.qLL(:,:) CR_GenStilde.qLL(:,:)];
        
test = ['GMM_S     ';
        'GMM_CLR   ';
        'ave_S     ';
        'exp_S     ';
        'qLL_S     ';
        'ave_Stilde';
        'exp_Stilde';
        'qLL_Stilde';
        'Split_S   ';
        'Split_CLR '];            
l.vrho = length(Vrho);        
ini = 1; fin=l.vrho;  
    
for i=1:10
    figure (i)
    contour(Vrho,Alpha,CR_values(:,ini:fin),'ShowText','on','LineColor',[0 0 0],...
        'LevelList',[0 0.9 0.95],...
        'LabelSpacing',250,...
        'Fill','on');
    colormap(Color1);
    grid on
    xlabel('$\varrho$','Interpreter','latex','fontsize',14,'fontweight','b')
    ylabel('$\alpha$','Interpreter','latex','fontsize',14,'fontweight','b')
    saveas(gcf,'CR_GMM_S_6610', 'pdf')
    ini=l.vrho*i+1;
    fin=ini+l.vrho-1;
end

%close all

toc

%--- Change the path according to your computer
rmpath('.../functions')
rmpath('.../tables')



