%--- New Keynesian Phillips Curve - baseline model.
%--- Confidence Regions using GMM weak instruments tests, Split sample 
%--- test (S, KLM,JKLM, KJ, CLR) and generalized S (sup, ave, exp, qll_10)
%--- We concentrate out the constant before implementing the test.
%--- See note: "Identification using stability restrictions: supplement "

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%.%    Hybrid  Model NKPC with time-varying trend inflation                %.%
%.% when beta =1; the moment condition is:                                 %.%
%.% E[ f_t + g_t*bar{pi}_t] = 0                                            %.%
%.%                                                                        %.%
%.% f_t = Z_t'*(e_t - alpha*e_{t+1}                                        %.%
%.% g_t = Z_t'*varpi_t                                                     %.%
%.%                                                                        %.%            
%.%   where:                                                               %.%
%.%                                                                        %.%
%.%  e_t   = (Vrho*diff(pi_t) - diff(pi_{t+1}) - gamma*upsilon*x_t - kappa)%.%   
%.% gamma  = (1-alpha)^2/alpha                                             %.%
%.% upsilon= a*(mu-1)/(mu-a)                                               %.%                
%.% kappa  = gamma*upsilon*ln(mu)                                          %.%
%.% varpi_t= (1-alpha)*lambda_pi*ln(mu) + Y_t*beta_pi +                    %.%
%.%          (1-alpha)*(1-Vrho)(r_{t,t+1}+diff(y_{t+1}) +                  %.%
%.%          vtheta*pi_{t+1}                                               %.%
%.%                                                                        %.%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

tic

%--- Add the folder functions and table in the search path
addpath('.../functions','-end')
addpath('.../tables','-end')

nkpc =importdata('nkpc_data.mat');	
range=[1984.1 2010.4];
%---adjustment factor labor share: upsilon = (a*(mu-1))/(mu-a);
a  = 2/3; % labor share - Cobb Douglas
oma= 1-a;
%% Parameter Space Grid Search Points   
Alpha   = 0.01:0.02:.99 ; 
l.alpha = length(Alpha) ;
Vrho    = 0.00:0.02:.99 ;               
l.vrho  = length(Vrho)  ;
Mu      = 1.01:0.01:1.40;  % markup
l.mu    = length(Mu)   ;

[X,Y,Z,Ypi,T,yearq] = data_nkpc_ti(nkpc,range,a);
%---matrix of dependent variables and instruments
Xp      = X'; 
Zp      = Z';
YBar    = [Y X];

%---Dimensions  
p      = 3;           % # of endogenous parameters
k.y    = size(Y,2);   % # of endogenous variables
k.x    = size(X,2);   % # of included instruments
k.z    = size(Z,2);   % # of instruments (total)
k.zm1  = k.z-1;
k.zmx  = k.z-k.x;       % # of excluded instruments
kmkx1  = k.z-k.x-1;
k.zz   = 2*k.z;
k.yy   = 2*k.y;
k.zzmx = k.zz-k.x;
k.zzmxx= k.zz-2*k.x;
kkmkx2 = k.zz-(k.x+2);
k.zzm2 = k.zz-2;
k.zzmxx= k.zz-2;
k.zy   = k.z*k.y;

%--- trimming paramter 
trim   = 0.15;    
t.l    = ceil(trim*T); t.u =floor((1-trim)*T);
s.s    = (t.l:1:t.u)/T;

%--- terms for qLL_AR test
c10 = 10; term_c10= c10/(1+c10);
%---Auxiliary Projection Matrices
I.T = eye(T);
Mx  = I.T - X*((Xp*X)\(Xp));

%--- HAC estimate of Sigma_hat (see equation (S-8))
scale    = T/(T-k.x);
Sigma_hat= scale*HAC_Sigma_hat(Mx*Y,Z,k.y,k.z,T);

%--- Permutation Matrices (computing Var(vec(s.ZY)), see equation (S-16)
% Per.C = kron(E ,I.k.z);  %permutation columns            
% Per.R = kron(E',I.k.z);  %permutation rows            
I.k.z        = eye(k.z);
[Per.C,Per.R]= permut_matrix(k.y,k.yy,I.k.z);

%--- Weak Instruments, Split Sample, Generalized S, and Stability Tests 
GMM   = struct('S',zeros(l.alpha,l.vrho,l.mu),'KLM',zeros(l.alpha,l.vrho,l.mu),...
      'JKLM',zeros(l.alpha,l.vrho,l.mu),'CLR',zeros(l.alpha,l.vrho,l.mu),'rk',zeros(l.alpha,l.vrho,l.mu));

Split = struct('S',zeros(l.alpha,l.vrho,l.mu), 'KLM',zeros(l.alpha,l.vrho,l.mu),...
      'JKLM',zeros(l.alpha,l.vrho,l.mu),'CLR',zeros(l.alpha,l.vrho,l.mu),'rk',zeros(l.alpha,l.vrho,l.mu));

GenS   = struct('ave', zeros(l.alpha,l.vrho,l.mu),...
      'exp',zeros(l.alpha,l.vrho,l.mu),'qLL',zeros(l.alpha,l.vrho,l.mu));

GenStilde  = struct('ave', zeros(l.alpha,l.vrho,l.mu),...
      'exp',zeros(l.alpha,l.vrho,l.mu),'qLL',zeros(l.alpha,l.vrho,l.mu));

T_break_hat= zeros(l.alpha,l.vrho,l.mu);

%---Matrices of Ones
ons.k.z   = ones(1,k.z); ons.k.y  = ones(1,k.y); ons.T = ones(T,1);

%---Identity matrices
I.k.z  = eye(k.z)     ; I.k.zmx  = eye(k.zmx)   ; I.kkm1 = eye(2*k.z-k.x) ;
I.k.zz = eye(k.zz)    ; I.p     = eye(p)      ; I.k.y   = eye(k.y)     ; I.k.yy= eye(2*k.y); 
I.k.zzmxx = eye(2*(k.z-1)); 
I.m1 = eye(p+1)   ; I.m1m1  = eye(2*(p+1)); I.k.zzmx= eye(2*k.z-k.x) ;
I.kmkx1=eye(kmkx1); I.kkmkx2=eye(kkmkx2)  ; I.kkm3 = eye(2*k.z-3)  ;   
DVD = zeros(l.alpha,l.vrho,l.mu,t.u-t.l+1);
%---Auxiliary Matrices
D_hat       = zeros(k.zmx,p)   ; s.D_hat   = zeros(k.zzmxx,p); 
seq.KLM   = zeros(1,t.u-t.l+1); seq.JKLM = zeros(1,t.u-t.l+1); 
s.S  = zeros(1,t.u-t.l+1); s.Stilde= zeros(1,t.u-t.l+1);  
s.DVD = zeros(1,t.u-t.l+1); s.s = (t.l:1:t.u)/T;

%% Loop: Hypothesis Testing on Parameter Space Grid Search Points 
for n=1:l.mu
    mu      = Mu(n);
    lmu     = log(mu);
    muma    =(mu-a);
    muma2   = muma^2;
    mumo    = mu-1;
    upsilon = (a*mumo)/muma;
    uplmu   = upsilon*lmu;
    dup_mu  = (a*oma)/(muma2);% derivative of upsilon wrt mu
       
    for j=1:l.vrho
        
        vrho   = Vrho(j);
        omvr = 1-vrho;
        omvr2= omvr.^2;
        for i=1:l.alpha
            
            alpha = Alpha(i);
            omal  = (1-alpha);
            omal2 = omal^2;
            gamma = omal2/alpha;
            kappa = gamma*uplmu;
            
            b    = [vrho; -(1+alpha*vrho); -gamma*upsilon; alpha; omal^2*upsilon];
            c    = omal*kappa;
            bBar = [b;-c];
            Yb   = Y*b;
            YbBar= YBar*bBar; % same as e_hat= Yb-Xc  see equation (S-27)

                        %--- Derivative of b w.r.t ``deep parameters''
            db   = [  1             0                                      0;
                      -alpha      -vrho                                    0;
                       0   -((alpha^2-1)/(alpha^2))*upsilon    -gamma*dup_mu;
                       0             1                                     0;
                       0       -2*omal*upsilon                  omal2*dup_mu];
            %--- Derivative of bBar w.r.t ``deep parameters''       
            dbBar = [db; 0 ((2*alpha+1)/alpha)*kappa -omal*gamma*(dup_mu*lmu + upsilon/mu)];
            
            b_bar  = kron(b,I.k.z);
            V_ff   = b_bar'*Sigma_hat*b_bar;% Variance S and qLL-S tests see equation (S-7)
            hV_ff  = sqrtm(V_ff);           % Symmetric square root
            
            %---Parameters trend inflation term
            
            aux       = (a+alpha*mu)./(a*mumo);
            vtheta    = (1/upsilon + 1/mumo)*omal*omvr2;
            
            lambda_pi = -(omal/alpha)*(omvr*aux)*upsilon;
            b_pi      = [0; (1/upsilon + omal/mumo)*omvr*vrho; lambda_pi; -(alpha*omvr)/upsilon;-alpha*lambda_pi];
            l_pi      = [omal*lambda_pi*lmu; b_pi;  omal*omvr; vtheta];    %---see equation (S-29)
            
            Gpi  = Zp*Ypi*l_pi;

            hGpi = hV_ff\Gpi; hG_pip = hGpi';
            %---Project out the trend inflation variables 
            M_hGpi = I.k.z - hGpi*(hG_pip*hGpi\hG_pip);  %---see equation (S-30)
           
            %--- Normalized moment condition
            F_hat              = ((Z.*(YbBar*ons.k.z))/hV_ff)*M_hGpi; % see equation (S-12)
            GenStilde.qLL(i,j,n)= qLL_S_tilde(F_hat,c10);
            
            %--- SVD of the orthogonal projection matrix
            [U, ~]= svd(M_hGpi);
            L     = U(1:k.z,1:k.zm1);  % L*L' = M_hGpi & L'*L = I(k.z-1)
            
            Xi_hat = L'*(hV_ff\(Zp*YbBar));
            Xi_hatp= Xi_hat';               % see statistic S_T on page 40
            
            GMM.S(i,j,n)   = (1/T)*Xi_hatp*Xi_hat;
            GenS.qLL(i,j,n)= term_c10*GMM.S(i,j,n) + GenStilde.qLL(i,j,n);
            
            %---Jacobian Matrix for KLM test
            L_tilde = hV_ff\L;
            
            Aux1  = kron(b,L_tilde);
            Aux2  = kron(db,L_tilde); % == db_bar*kron(I.p,L_tilde);
            V_xx  = I.k.zmx;            % == Aux1'*Sigma_hat*Aux1;
            V_qx  = Aux2'*Sigma_hat*Aux1;
            V_qq  = Aux2'*Sigma_hat*Aux2;
            V_qqx = V_qq - V_qx*(V_xx\V_qx');
            
            q_hat = L_tilde'*Zp*YBar*dbBar; h = 0;
            
            for r=1:p
                D_hat(:,h+1:r)= q_hat(:,r)...
                    - V_qx( k.zmx*h+1: k.zmx*r,:)*(V_xx\(Xi_hat));
                h=h+1;
            end
            D_hat = D_hat/T;
            D_hatp= D_hat';
            
            GMM.KLM(i,j,n) = (1/T)*Xi_hatp*D_hat*((D_hatp*D_hat)\(D_hatp*(Xi_hat)));
            GMM.JKLM(i,j,n)= GMM.S(i,j,n)-GMM.KLM(i,j,n);
            
            %rank statistic for Conditional LR test:
            GMM.rk(i,j,n)  = ranktest_svd(D_hat,I.p, V_xx^(-.5),V_qqx,p-1,T);
            GMM.CLR(i,j,n) = .5*(GMM.S(i,j,n)-GMM.rk(i,j,n)+...
                sqrt((GMM.S(i,j,n)+GMM.rk(i,j,n))^2 - 4*GMM.JKLM(i,j,n)*GMM.rk(i,j,n)));
            
            %--- Loop for computing Split-sample S && estimation of break date
            s.b_bar = kron(b,I.k.zz);
            s.b_barp= s.b_bar';
            s.GpiGpi= blkdiag(Gpi,Gpi);
            for t_break  = t.l:t.u
                ts   = t_break-t.l+1;
                s.Z  = blkdiag(Z(1:t_break,:),Z(t_break+1:T,:)); % see equation (S-13)
                s.Zp = s.Z';
                st    = s.s(ts);
                %--- DANGER: s.Sigma_hat != var(s.Zp*Y)
                s.Sigma_hat = blkdiag(st*Sigma_hat,(1-st)*Sigma_hat);
                s.Sigma_hat =  Per.R*s.Sigma_hat*Per.C; % see equation (S-16)
                %---- Var( T^-.5 Zbar*Yb )
                s.V_ff=  s.b_barp*s.Sigma_hat*s.b_bar; % see equation (S-16)
                s.hV_ff = sqrtm(s.V_ff);
                
                s.hGpi  = s.hV_ff\s.GpiGpi;
                s.hG_pip  = s.hGpi';
                %--- SVD of the orthogonal projection matrix
               
                s.M_hGpi = I.k.zz - s.hGpi*(s.hG_pip*s.hGpi\s.hG_pip);
                
                [s.U, s.eig]=svd(s.M_hGpi);
                s.L  = s.U(1:k.zz,1:k.zzm2);
                
                s.X_T    = s.hV_ff\(s.Zp*YbBar);
                s.Xi_hat = s.L'*s.X_T;
                s.Xi_hatp= s.Xi_hat';
				
                s.S(ts)     = (1/T)*s.Xi_hatp*(s.Xi_hat);
                s.Stilde(ts)= s.S(ts)-GMM.S(i,j,n);
                
                s.L_tilde= s.hV_ff\s.L;
                s.Aux1   = kron( b,s.L_tilde);
                s.Aux2   = kron(db,s.L_tilde);
                
                s.V_xx = I.k.zzmxx;  %==s.Aux1'*s.Sigma_hat*s.Aux1; (identity matrix)
                s.V_qx = s.Aux2'*s.Sigma_hat*s.Aux1;
                s.V_qq = s.Aux2'*s.Sigma_hat*s.Aux2;
                s.V_qqx= s.V_qq - s.V_qx*(s.V_xx\s.V_qx');
                
                s.q_hat= s.L_tilde'*s.Zp*YBar*dbBar; h = 0;
                
                for r=1:p
                    s.D_hat(:,h+1:r)= s.q_hat(:,r)-...
                        s.V_qx((k.zzmxx)*h+1:(k.zzmxx)*r,:)...
                        *(s.V_xx\s.Xi_hat);
                    h=h+1;
                end
                s.D_hat = s.D_hat/T;
                s.D_hatp = s.D_hat';
                
                s.KLM(ts)=(1/T)*...
                    s.Xi_hatp*s.D_hat*((s.D_hatp*s.D_hat)\(s.D_hatp*s.Xi_hat));
                s.DVD(ts)= s.D_hat(:)'*(s.V_qqx\s.D_hat(:));
                
                % Saved s.D_hat and s.V_qqx to be used in computing split CLR
                s.D(ts,:,:)  = s.D_hat;
                s.V_D(ts,:,:)= s.V_qqx;
            end
            GenStilde.ave(i,j,n) = mean(s.Stilde);
            GenStilde.exp(i,j,n) = log(mean(exp(.5*s.Stilde)));
            
            GenS.ave(i,j,n) = mean(s.S);
            GenS.exp(i,j,n) = log(mean(exp(.5*s.S)));
            %---Estimating the break date
            [~, ir]           = sort(s.DVD,'descend');
            t_break_hat       = t.l+ir(1)-1;
            T_break_hat(i,j,n)= yearq(t_break_hat,1);
           
            Split.S(i,j,n)   = s.S(ir(1));
            Split.KLM(i,j,n) = s.KLM(ir(1));
            Split.JKLM(i,j,n)= Split.S(i,j,n)-Split.KLM(i,j,n);
            
            sD_hat = squeeze(s.D(ir(1),:,:));
            sV_qqx = squeeze(s.V_D(ir(1),:,:));
            
            rkt =  ranktest_svd(sD_hat,I.p, s.V_xx^(-.5),sV_qqx,p-1,T);
            Split.rk(i,j,n) = rkt;
            Split.CLR(i,j,n)= .5*(Split.S(i,j,n)-rkt+ ...
                     sqrt((Split.S(i,j,n) + rkt)^2 - 4*Split.JKLM(i,j,n)*rkt));

        end
    end
end
%% Computing the p-values of tests

%--- P-values: Weak Instruments Tests
CR_GMM = struct(...
        'S',chi2cdf(GMM.S(:,:,:),k.z-1), 'KLM',chi2cdf(GMM.KLM(:,:,:),p),...
         'JKLM',chi2cdf(GMM.JKLM(:,:,:),k.z-1-p), 'CLR',zeros(l.alpha,l.vrho,l.mu));
%--- P-values: Split Sample Tests
CR_Split = struct(...
        'S',chi2cdf(Split.S(:,:,:),k.zz-2), 'KLM',chi2cdf(Split.KLM(:,:,:),p),...
         'JKLM',chi2cdf(Split.JKLM(:,:,:),k.zz-2-p), 'CLR',zeros(l.alpha,l.vrho,l.mu));

%--- Space for Generalized-S and Stability-S Tests p-values
CR_GenS = struct('ave',zeros(l.alpha,l.vrho,l.mu),...
                'exp',zeros(l.alpha,l.vrho,l.mu),'qLL',zeros(l.alpha,l.vrho,l.mu));
CR_GenStilde= struct('ave',zeros(l.alpha,l.vrho,l.mu),...
                'exp',zeros(l.alpha,l.vrho,l.mu),'qLL',zeros(l.alpha,l.vrho,l.mu));
            
P_values1    = pvalue_CLR(p,k.zm1,l.alpha*l.vrho*l.mu,GMM.CLR(:),GMM.rk(:));
CR_GMM.CLR   = reshape(P_values1, [l.alpha l.vrho l.mu]);

P_values2    = pvalue_CLR(p,k.zzm2,l.alpha*l.vrho*l.mu,Split.CLR(:),Split.rk(:));
CR_Split.CLR = reshape(P_values2, [l.alpha l.vrho l.mu]);

%--- P-values: Generalized-S and S_tilde Tests
[P_values_genS, P_values_genStilde] = pvalue_GenS(k.zm1,k.zm1,l.alpha*l.vrho*l.mu,GenS,GenStilde);
CR_GenS.ave = reshape(P_values_genS(:,1),[l.alpha l.vrho l.mu]);
CR_GenS.exp = reshape(P_values_genS(:,2),[l.alpha l.vrho l.mu]);
CR_GenS.qLL = reshape(P_values_genS(:,3),[l.alpha l.vrho l.mu]); 

CR_GenStilde.ave = reshape(P_values_genStilde(:,1),[l.alpha l.vrho l.mu]);
CR_GenStilde.exp = reshape(P_values_genStilde(:,2),[l.alpha l.vrho l.mu]);
CR_GenStilde.qLL = reshape(P_values_genStilde(:,3),[l.alpha l.vrho l.mu]);

test = ['GMM-S     ';
        'GMM-CLR   ';
        'ave-S     ';
        'exp-S     ';
        'qLL-S     ';
        'ave-Stilde';
        'exp-Stilde';
        'qLL-Stilde';
        'Split-S   ';
        'Split-CLR '];

Level = [0.95 0.90 0.85 0.80];
nobs = length(CR_GMM.S(:));
ll    = length(Level);

Vol.GMM = struct('S',zeros(1,ll), 'KLM',zeros(1,ll),...
                 'JKLM',zeros(1,ll),'CLR',zeros(1,ll));
   
Vol.Split= struct('S',zeros(1,ll), 'KLM',zeros(1,ll),...
                 'JKLM',zeros(1,ll),'CLR',zeros(1,ll));
				 
Vol.GenS     = struct('ave',zeros(1,ll),...
                  'exp',zeros(1,ll),'qLL',zeros(1,ll));
Vol.GenStilde= struct('ave',zeros(1,ll),...
                'exp',zeros(1,ll),'qLL',zeros(1,ll));

for i=1:ll
    Vol.GMM.S(i)         = sum(CR_GMM.S(:)<=Level(i))./nobs;
    Vol.GMM.CLR(i)       = sum(CR_GMM.CLR(:)<=Level(i))./nobs;
    Vol.Split.S(i)       = sum(CR_Split.S(:)<=Level(i))./nobs;
    Vol.Split.CLR(i)     = sum(CR_Split.CLR(:)<=Level(i))./nobs;
    Vol.GenS.ave(i)      = sum(CR_GenS.ave(:)<=Level(i))./nobs;
    Vol.GenS.exp(i)      = sum(CR_GenS.exp(:)<=Level(i))./nobs;
    Vol.GenS.qLL(i)      = sum(CR_GenS.qLL(:)<=Level(i))./nobs;
    Vol.GenStilde.ave(i) = sum(CR_GenStilde.ave(:)<=Level(i))./nobs;
    Vol.GenStilde.exp(i) = sum(CR_GenStilde.exp(:)<=Level(i))./nobs;
    Vol.GenStilde.qLL(i) = sum(CR_GenStilde.qLL(:)<=Level(i))./nobs;
end


fprintf(1,'(Volume of Confidence Region)/(Volume of the Cube, %2.00f obs.)\n',T)
fprintf(1,'                 (in percentage)                   \n')
fprintf(1,'tests        %4.0f%%  %4.0f%%  %4.0f%%  %4.0f%% \n\n',...
          100*Level(1), 100*Level(2), 100*Level(3),100*Level(4))

fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(1,:) , 100*Vol.GMM.S(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(2,:) , 100*Vol.GMM.CLR(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(3,:) , 100*Vol.GenS.ave(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(4,:) , 100*Vol.GenS.exp(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(5,:) , 100*Vol.GenS.qLL(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f| \n',...
    test(6,:) ,100*Vol.GenStilde.ave(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(7,:) ,100*Vol.GenStilde.exp(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(8,:) ,100*Vol.GenStilde.qLL(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(9,:) ,100*Vol.Split.S(:))
fprintf(1,'%s | %5.2f  %5.2f  %5.2f  %5.2f|  \n',...
    test(10,:) ,100*Vol.Split.CLR(:))


%--- Save the values to produce the figures  
save data_nkpc_trend_infl_8410 Vrho Alpha Mu CR_* yearq T_break_hat 


[X1,Y1,Z1] = meshgrid(Vrho,Alpha,Mu);

CR_values = [CR_GMM.S(:,:,:) CR_Split.S(:,:,:) CR_GMM.CLR(:,:,:) CR_Split.CLR(:,:,:) ...
            CR_GenS.ave(:,:,:) CR_GenStilde.ave(:,:,:) CR_GenS.exp(:,:,:)...
            CR_GenStilde.exp(:,:,:) CR_GenS.qLL(:,:,:) CR_GenStilde.qLL(:,:,:)];

test = ['GMM_S     ';
        'GMM_CLR   ';
        'ave_S     ';
        'exp_S     ';
        'qLL_S     ';
        'ave_Stilde';
        'exp_Stilde';
        'qLL_Stilde';
        'Split_S   ';
        'Split_CLR '];        

ini = 1; fin=l.vrho;        
for i=1:10
    
    DD=smooth3(1-CR_values(:,ini:fin,:));
    figure1= figure(i);
    axes1 = axes('Parent',figure1,...
        'ZTickLabel',{'1','1.05','1.10','1.15','1.20','1.25','1.30','1.35','1.40'},...
        'YTickLabel',{'0','0.25','0.50','0.75','1'},...
        'YTick',[0 0.25 0.5 0.75 1],...
        'XTickLabel',{'0','0.25','0.50','0.75','1'},...
        'XTick',[0 0.25 0.5 0.75 1],...
        'PlotBoxAspectRatio',[1 1 1],...
        'CameraViewAngle',8.3);
    view(axes1,[-40 14]);
    patch(isosurface(X1,Y1,Z1,DD,.05),...
        'Parent',axes1,...
        'FaceColor',[.9 .9 .9],...
        'EdgeColor','none');
    patch(isocaps(X1,Y1,Z1,DD,.05), ...
        'Parent',axes1,...
        'FaceColor',[0.80 0.80 0.80],...
        'EdgeColor','none');
    grid on
    colormap(gray)
    xlim([0 1]); ylim([0 1]); zlim([1 1.4])
    light('Parent',axes1,...
        'Position',[0.0 -0.2 .8]);
    xlabel('$\varrho$','Interpreter','latex','fontsize',14,'fontweight','b')
    ylabel('$\alpha$','Interpreter','latex','fontsize',14,'fontweight','b')
    zlabel('$\mu$','Interpreter','latex','fontsize',14,'fontweight','b')
    saveas(gcf,test(i,:), 'pdf')
    ini=l.vrho*i+1;
    fin=ini+l.vrho-1;
    
end
        
% close all

toc

%--- Change the path according to your computer
rmpath('.../functions')
rmpath('.../tables')
