function [X,Y,Z,T,yearq] = data_nkpc(nkpc,range, mu,a)
% This function returns the variables for the nkpc model
% nkpc  : data  from 1964-3 to 2011-2 (see explanation below)
% range : 1 x 2 vector with the initial and end period
% mu    : mark-up   
% a     : parameter of labor in the Cobb-Douglas Production Function
%
% Output
% X    : matrix of exogenous explanatory variables
% Y    : matrix of covariates
% Z    : matrix of instruments
% T    : number os observations
% yearq: year quarter

yquarter   = nkpc(:,1)   ; % year.quarter 1965-1 to 2011-2
GDPDEF     = nkpc(:,2)   ; % 1964-3 to 2011-2
FEDFUNDS   = nkpc(:,3)   ; % 1964-3 to 2011-2 3-month average
RGDPNFB    = nkpc(:,4)   ; % 1964-3 to 2011-2
LSHARENFB  = nkpc(:,5)   ; % 1964-3 to 2011-2 labor share nonfarm business sector in percentage

T_0=sum((yquarter<=range(1,1)));
T_f=sum((yquarter<=range(1,2)));

yearq = yquarter(T_0:T_f,1);
T     = length(yearq);          % # of observations

upsilon = (a*(mu-1))/(mu-a);

MC   = upsilon*log([LSHARENFB(T_0+1:T_f+1,1)/(100*a)...
                    LSHARENFB(T_0+0:T_f+0,1)/(100*a)...
                    LSHARENFB(T_0-1:T_f-1,1)/(100*a)...
                    LSHARENFB(T_0-2:T_f-2,1)/(100*a)...
                    LSHARENFB(T_0-3:T_f-3,1)/(100*a)]);
     
INF  = log([GDPDEF(T_0+2:T_f+2,1)./GDPDEF(T_0+1:T_f+1,1)...
            GDPDEF(T_0+1:T_f+1,1)./GDPDEF(T_0+0:T_f+0,1)...
            GDPDEF(T_0+0:T_f+0,1)./GDPDEF(T_0-1:T_f-1,1)...
            GDPDEF(T_0-1:T_f-1,1)./GDPDEF(T_0-2:T_f-2,1)...
            GDPDEF(T_0-2:T_f-2,1)./GDPDEF(T_0-3:T_f-3,1)...
            GDPDEF(T_0-3:T_f-3,1)./GDPDEF(T_0-4:T_f-4,1)...
            GDPDEF(T_0-4:T_f-4,1)./GDPDEF(T_0-5:T_f-5,1)...
            GDPDEF(T_0-5:T_f-5,1)./GDPDEF(T_0-6:T_f-6,1)]);
     
DINF = INF(:,1:7)-INF(:,2:8); % matrix of lagged diff inflation

Y = [DINF(:,3) DINF(:,2) MC(:,2)]; % [dinf_{t} dinf_{t+1} xtilde_t];
X = ones(T,1);                     % constant vector
Z = [X DINF(:,4:5) MC(:,3:5)];     % matrix of instruments

end