function [P_values_genS, P_values_genStilde] = pvalue_GenS(k_z,k_zmx,L,GenS,GenStilde)
%pvalue_CLR returns the p-value for CLR tests
%   k_z      = number of instruments
% k_zmx      = number of overidentification restrictions
%          L = number of grid points  
%    GenS    = Gen S tatistics  (struct entry) 
%  GenStilde = Gen S tilde statistics (struct entry)


%---Table: Critical values for GenS-S and GenStilde-S tests
tabCV.ave   = importdata('Table_bb_ave.mat');
tabCV.exp   = importdata('Table_bb_exp.mat');

pp=1;   %  if trim=.15, then pp=1; if trim=.2, then pp=2;
CV.ave  = squeeze(tabCV.ave(pp,:,:));
CV.exp  = squeeze(tabCV.exp(pp,:,:));
CV.qLL  = importdata('table_qLLS_c10.mat');

clear tabCV

stab    = size(CV.ave,2);
r_bs    = 1+ sum(1:k_z)+ (k_zmx);     % row position on the c.v. table
TabGen  = struct('ave', [CV.ave(1,1:stab) ; CV.ave(r_bs,1:stab) ],...
                 'exp', [CV.exp(1,1:stab) ; CV.exp(r_bs,1:stab) ],...
                 'qLL', [CV.qLL(1,1:stab) ; CV.qLL(r_bs,1:stab)]);

r_bb     = 1+ sum(1:k_z);
TabStab = struct('ave', [CV.ave(1,1:stab) ; CV.ave(r_bb,1:stab) ],...
                 'exp', [CV.exp(1,1:stab) ; CV.exp(r_bb,1:stab) ],...
                 'qLL', [CV.qLL(1,1:stab) ; CV.qLL(r_bb,1:stab)]);

clear CV r_* stab 

GenS_ave=GenS.ave(:);
GenS_exp=GenS.exp(:);
GenS_qLL=GenS.qLL(:);

GenStilde_ave=GenStilde.ave(:);
GenStilde_exp=GenStilde.exp(:);
GenStilde_qLL=GenStilde.qLL(:);

P_GenS     = zeros(L,3);
P_GenStilde= zeros(L,3);
for j=1:L
    
    ind_q = 2;
    while (TabGen.ave(2,ind_q)<= GenS_ave(j) && ind_q<=999)
        ind_q=ind_q+1;
    end
    P_GenS(j,1) = TabGen.ave(1,ind_q);
    
    ind_q = 2;
    while (TabGen.exp(2,ind_q)<= GenS_exp(j) && ind_q<=999)
        ind_q=ind_q+1;
    end
    P_GenS(j,2) = TabGen.exp(1,ind_q);
    
    ind_q = 2;
    while (TabGen.qLL(2,ind_q)<= GenS_qLL(j) && ind_q<=999)
        ind_q=ind_q+1;
    end
    P_GenS(j,3) = TabGen.qLL(1,ind_q);
    
    ind_q = 2;
    while (TabStab.ave(2,ind_q)<= GenStilde_ave(j) && ind_q<=999)
        ind_q=ind_q+1;
    end
    P_GenStilde(j,1) = TabStab.ave(1,ind_q);
    
    ind_q = 2;
    while (TabStab.exp(2,ind_q)<= GenStilde_exp(j) && ind_q<=999)
        ind_q=ind_q+1;
    end
    P_GenStilde(j,2) = TabStab.exp(1,ind_q);
    
    ind_q = 2;
    while (TabStab.qLL(2,ind_q)<= GenStilde_qLL(j) && ind_q<=999)
        ind_q=ind_q+1;
    end
    P_GenStilde(j,3) = TabStab.qLL(1,ind_q);
    
end

P_values_genS     = P_GenS;
P_values_genStilde= P_GenStilde;

end

