function qLL = qLL_S_tilde(mU,c_bar)
    % qLL_AR_stat computes the stab-AR(c_bar)-test 
    % stab-AR(c_bar)-test is stability part of the qLL-AR test proposed 
    % in Mavroeidis and Magnusson (2010)
    % 
    % qLL_AR = GMM-AR + stab-AR(c_bar)
    %
    % stab-AR(c_bar) = v'*M_e*v - v'*G_c*v
    %
    % Parameters:
    % mU    == T x k matrix of scale moment functions
    % c_bar == scale parameter (measures the magnitude of the break)

    T=size(mU,1); 
    
    mUbar = mU-ones(T,1)*mean(mU);       

    rtilde= 1 - c_bar/T; 
    ini   = [mU(1,:); diff(mU)];
    c     = [1;cumprod(ones(T-1,1).*rtilde)];
    r     = [1 zeros(1,T-1)];
    mw    = toeplitz(c,r)*ini;
    vr    = rtilde.^(1:1:T)';
    b     = (vr'*vr)\(vr'*mw);
    e     = mw - vr*b;
    e2    = sum(sum(e.^2,1),2);    
    qLL   = sum(sum(mUbar.^2),2) - e2*rtilde; 
    %       v'*G_c*v  - v'*M_e*v
end
